package net.rumati.logging.muffero;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import net.rumati.logging.muffero.appender.filter.PriorityFilterAppender;

public abstract class Configurator
{
    private final HashMap<PriorityFilterAppender, List<Logger>> configuredAppenders = new HashMap<PriorityFilterAppender, List<Logger>>();

    protected synchronized void addAppender(Logger logger, Appender appender, Priority priority)
    {
        PriorityFilterAppender fa = new PriorityFilterAppender(appender, priority);
        List<Logger> list = configuredAppenders.get(fa);
        if (list == null){
            list = new LinkedList<Logger>();
            configuredAppenders.put(fa, list);
        }
        list.add(logger);
        logger.addAppender(fa);
    }

    public abstract void configure()
        throws LoggingException;

    public synchronized void deconfigure()
    {
        for (Entry<PriorityFilterAppender, List<Logger>> e : configuredAppenders.entrySet()){
            PriorityFilterAppender appender = e.getKey();
            for (Logger logger : e.getValue()){
                logger.removeAppender(appender);
            }
            appender.shutdown();
        }
    }
}
