package net.rumati.logging.muffero;

import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import net.rumati.logging.muffero.util.ReadWriteLockedMap;

public class LoggerFactory
{
    private static final ReadWriteLockedMap<String, Logger> loggers;
    private static final Logger rootLogger;
    private static final ReadWriteLock lock;

    static {
        lock = new ReentrantReadWriteLock(true);
        loggers = new ReadWriteLockedMap<String, Logger>(new TreeMap<String, Logger>(), lock);
        rootLogger = new Logger(null, "ROOT");
        rootLogger.setInheritParentAppenders(false);
    };
    
    private LoggerFactory(){}

    public static Logger getLogger(Class clazz)
    {
        return getLogger(clazz.getName());
    }

    public static Logger getLogger(String name)
    {
        Logger l = loggers.get(name);
        if (l == null){
            String parentName = name;
            Logger parent = null;
            lock.writeLock().lock();
            try {
                while (parent == null){
                    int idx = parentName.lastIndexOf(".");
                    if (idx < 0){
                        parent = rootLogger;
                    }else{
                        parentName = parentName.substring(0, idx);
                        parent = loggers.get(parentName);
                    }
                }
                l = new Logger(parent, name);
                loggers.put(name, l);
                lock.readLock().lock();
            }finally{
                lock.writeLock().unlock();
            }
            try {
                String childPrefix = name + ".";
                for (Entry<String, Logger> le : loggers.entrySet()){
                    if (le.getKey().startsWith(childPrefix)){
                        le.getValue().setParent(l);
                    }
                }
            }finally{
                lock.readLock().unlock();
            }
        }
        return l;
    }
    
    public static Logger getRootLogger()
    {
        return rootLogger;
    }
}
