package net.rumati.logging.muffero.appender;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import net.rumati.logging.muffero.Layout;

public class FileAppender
    extends WriterAppender
{
    private final Layout layout;
    private final transient Writer writer;
    private final transient File file;
    private final transient Charset encoding;

    public FileAppender(File file, Charset encoding, Layout layout) throws FileNotFoundException
    {
        this.layout = layout;
        this.file = file;
        this.encoding = encoding;
        this.writer = new OutputStreamWriter(new FileOutputStream(file, true), encoding);
    }

    public FileAppender(File logFile, Layout layout) throws FileNotFoundException
    {
        this(logFile, Charset.forName("UTF-8"), layout);
    }

    @Override
    protected Writer getWriter()
            throws IOException
    {
        return writer;
    }

    @Override
    protected Layout getLayout()
    {
        return layout;
    }

    public void shutdown()
    {
        if (writer != null){
            try {
                writer.close();
            }catch (IOException e){
                // TODO: log this somewhere
            }
        }
    }
}
