package net.rumati.logging.muffero.config.prop;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.util.ClassUtil;

public class PropertyConfigContext
{
    private final List<PropertyObjectLoader> loaders;

    PropertyConfigContext(List<PropertyObjectLoader> loaders)
    {
        this.loaders = loaders;
    }

    public <T> T loadObject(String className, Class<T> typeClass)
            throws LoggingException
    {
        return loadObjectWithConfigPrefix(className, typeClass, "", new TreeMap<String, String>());
    }

    public <T> T loadObjectWithConfigPrefix(String className, Class<T> typeClass, String configPrefix, Map<String, String> config)
            throws LoggingException
    {
        Map<String, String> resultConfig = new TreeMap<String, String>();
        for (Map.Entry<String, String> e : config.entrySet()){
            if (e.getKey().startsWith(configPrefix)){
                resultConfig.put(e.getKey().substring(configPrefix.length()), e.getValue());
            }
        }

        for (PropertyObjectLoader loader : loaders){
            if (loader.canLoad(className, typeClass)){
                return loader.loadObject(className, typeClass, resultConfig, this);
            }
        }

        try {
            Class c = Class.forName(className);
            if (!ClassUtil.classImplements(c, typeClass)){
                throw new LoggingException("Requested class \"" + className + "\" does not implement \"" + typeClass.getName() + "\"");
            }
            @SuppressWarnings("unchecked")
            Class<? extends T> ctc = c;
            try {
                return ctc.newInstance();
            }catch (IllegalAccessException e){
                throw new LoggingException("Error instantiating object " + className, e);
            }catch (InstantiationException e){
                throw new LoggingException("Error instantiating object " + className, e);
            }catch (ExceptionInInitializerError e){
                throw new LoggingException("Error instantiating object " + className, e);
            }
        }catch (ClassNotFoundException e){
            throw new LoggingException(e);
        }
    }
}