package net.rumati.logging.muffero.util;

import java.util.Collection;
import java.util.Iterator;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class ReadWriteLockedCollection<E>
        implements Collection<E>
{
    private final Collection<E> collection;
    protected final ReadWriteLock lock;

    public ReadWriteLockedCollection(Collection<E> values)
    {
        this(values, new ReentrantReadWriteLock(true));
    }

    public ReadWriteLockedCollection(Collection<E> values, ReadWriteLock lock)
    {
        this.collection = values;
        this.lock = lock;
    }

    public int size()
    {
        lock.readLock().lock();
        try{
            return collection.size();
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean isEmpty()
    {
        lock.readLock().lock();
        try{
            return collection.isEmpty();
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean contains(Object o)
    {
        lock.readLock().lock();
        try{
            return collection.contains(o);
        }finally{
            lock.readLock().unlock();
        }
    }

    public Iterator<E> iterator()
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedIterator<E>(collection.iterator(), lock);
        }finally{
            lock.readLock().unlock();
        }
    }

    public Object[] toArray()
    {
        lock.readLock().lock();
        try{
            return collection.toArray();
        }finally{
            lock.readLock().unlock();
        }
    }

    public <T> T[] toArray(T[] a)
    {
        lock.readLock().lock();
        try{
            return collection.toArray(a);
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean add(E e)
    {
        lock.writeLock().lock();
        try{
            return collection.add(e);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public boolean remove(Object o)
    {
        lock.writeLock().lock();
        try{
            return collection.remove(o);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public boolean containsAll(Collection<?> c)
    {
        lock.readLock().lock();
        try{
            return collection.containsAll(c);
        }finally{
            lock.readLock().unlock();
        }
    }

    public boolean addAll(Collection<? extends E> c)
    {
        lock.writeLock().lock();
        try{
            return collection.addAll(c);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public boolean removeAll(Collection<?> c)
    {
        lock.writeLock().lock();
        try{
            return collection.removeAll(c);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public boolean retainAll(Collection<?> c)
    {
        lock.writeLock().lock();
        try{
            return collection.retainAll(c);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public void clear()
    {
        lock.writeLock().lock();
        try{
            collection.clear();
        }finally{
            lock.writeLock().unlock();
        }
    }

    public void readLock()
    {
        lock.readLock().lock();
    }

    public void readUnlock()
    {
        lock.readLock().unlock();
    }
}